@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2005  The PARI group.
+Copyright (C) 2000-2013  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -10,7 +10,7 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #include "config.h"
 #include <stdlib.h>
@@ -19,9 +19,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include <ctype.h>
 #include "header.h"
 
+static int multi=0;
 static int newprivvar=1;
 extern int initnode;
 extern int optstrict;
+extern int linecount;
+extern int safecoeff;
 
 int newdecl(int flag, int t, int initval, int *v)
 {
@@ -35,14 +38,24 @@ int newcall(const char *s, int y)
   return newnode(Ffunction,newentry(s),y);
 }
 
-int listtoseq(int *stack, int nb)
+int stacktofun(int fun, int *stack, int nb)
 {
   int n,i;
   if (nb==-1) return -1;
-  for(n=stack[0],i=1;i<nb;n=newnode(Flistarg,n,stack[i++]));
+  for(n=stack[0],i=1;i<nb;n=newnode(fun,n,stack[i++]));
   return n;
 }
 
+int stacktoargs(int *stack, int nb)
+{
+  return stacktofun(Flistarg,stack,nb);
+}
+
+int stacktoseq(int *stack, int nb)
+{
+  return stacktofun(Fseq,stack,nb);
+}
+
 int newseq(int x, int y)
 {
   int seq;
@@ -122,11 +135,11 @@ int geninsertvar(int seq, int ret)
   {
     if (tree[seq].f==Fseq)
     {
-      bseq=newnode(Faffect,ret,tree[seq].y);
+      bseq=newnode(Fassign,ret,tree[seq].y);
       bseq=newnode(Fseq,tree[seq].x,bseq);
     }
     else
-      bseq=newnode(Faffect,ret,seq);
+      bseq=newnode(Fassign,ret,seq);
   }
   else
     bseq=seq;
@@ -157,7 +170,18 @@ int newcoeff(int n, int x, int y)
   int arg[3];
   int nb=y<0?2:3;
   arg[0]=n; arg[1]=x; arg[2]=y;
-  return newnode(Ffunction,OPcoeff,listtoseq(arg,nb));
+  return newnode(Ffunction,OPcoeff,stacktoargs(arg,nb));
+}
+
+int newcoeffsafe(int n, int x, int y)
+{
+  int arg[3];
+  int nb=y<0?2:3;
+  arg[0]=n; arg[1]=x; arg[2]=y;
+  if (!safecoeff)
+    return newnode(Ffunction,OPcoeff,stacktoargs(arg,nb));
+  else
+    return newcall("_safecoeff",stacktoargs(arg,nb));
 }
 
 void makeblock(int bl, int n, int aseq, int ret, int savx)
@@ -195,26 +219,11 @@ void makeblocks3(int bl1, int bl2, int bl3, int n, int bseq, int aseq, int ret,
   restorectx(sav0);
 }
 
-void genblock_nolabel(int n, int p)
-{
-  int l=newlabel(0);
-  genblock(n,p);
-  stack_pop_safe(&s_label,l);
-}
-
-Clabel_t genblock_label(int n, int p)
-{
-  Clabel_t t;
-  int l=newlabel(1);
-  genblock(n,p);
-  t=label[l];
-  stack_pop_safe(&s_label,l);
-  return t;
-}
-static void affectval(int vn, int val, int *seq)
+void affectval(int vn, int val, int *seq)
 {
   ctxvar *v = ctxstack+vn;
-  int valn=newnode(Faffect,newleaf(v->node),val);
+  if (val<0) die(-1,"Internal error: affectval");
+  int valn=newnode(Fassign,newleaf(v->node),val);
   *seq=addseqright(*seq,valn);
 }
 
@@ -243,7 +252,11 @@ int genblockdeclaration(int args, int n, int flag, int type, int *seq)
         /*Make sure GEN objects are gerepilable.*/
         val = newnode(Ftag, newnode(Ftag, newsmall(0), Ggen), tv);
       if (decl==global)
-        fillvar(var,flag,tv,val);
+      {
+        int f=fillvar(var,flag,tv,-1);
+        if (autogc && ctype[tv]==Vgen)
+          affectval(f,val,seq);
+      }
       else
         pushvar(var,flag,tv,val);
       break;
@@ -262,10 +275,12 @@ int genblockdeclaration(int args, int n, int flag, int type, int *seq)
           val=newcall("_const_quote",newstringnode(entryname(stack[i]),-1));
           affectval(fillvar(stack[i],flag,mint,-1),val,seq);
         }
-        else if (autogc)
+        else
         {
+          int f = fillvar(stack[i],flag,mint,-1);
           /*Make sure (implicitly GEN) objects are gerepilable.*/
-          fillvar(stack[i],flag,mint, newsmall(0));
+          if (autogc)
+            affectval(f,  newsmall(0), seq);
         }
         break;
       case function:
@@ -275,31 +290,70 @@ int genblockdeclaration(int args, int n, int flag, int type, int *seq)
         break;
       }
       break;
-    case Faffect:
-      genequal(stack[i],"declaration",&var,&val,&tv);
-      if (tv==Gnotype)
-        tv=mint;
+    case Fassign:
       switch (decl)
       {
       case global:
+        genequal(stack[i],"declaration",&var,&val,&tv);
+        if (tv==Gnotype)
+          tv=mint;
         affectval(fillvar(var,flag,tv,-1),val,seq);
+        genblock(val,n);
         break;
       case local:
-        if (tree[val].f==Fsmall)
-          pushvar(var,flag,tv,val);
-        else
-          affectval(pushvar(var,flag,tv,-1),val,seq);
+        {
+          int x = tree[stack[i]].x;
+          if (tree[x].f==Fvec)
+          {
+            int vars[STACKSZ];
+            int j, ret;
+            int nb=listtostack(tree[x].x,Fmatrixelts,vars,STACKSZ," declaration",n);
+            val = tree[stack[i]].y;
+            newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&ret);
+            *seq = addseqright(*seq, geninsertvar(val,ret));
+            for (j=0;j<nb;j++)
+            {
+              checkisvar(vars[j],"my",&var,&tv);
+              if (tv==Gnotype)
+                tv=mint;
+              affectval(pushvar(var,flag,tv,-1),newcoeff(newleaf(ret),newsmall(j+1),-1),seq);
+            }
+          } else
+          {
+            genequal(stack[i],"declaration",&var,&val,&tv);
+            if (tv==Gnotype)
+              tv=mint;
+            if (tree[val].f==Fsmall)
+              pushvar(var,flag,tv,val);
+            else
+              affectval(pushvar(var,flag,tv,-1),val,seq);
+          }
+          genblock(val,n);
+        }
         break;
       case function:
         {
-          int vn = pushvar(var,flag,tv,val);/* can change ctxstack */
-          ctxvar *v = ctxstack+vn;
-          if (decl==function && descfindrules1(v->node,FC_default_check))
-            v->flag|=(1<<Cdefmarker);
+          int vn;
+          ctxvar *v;
+          genequal(stack[i],"declaration",&var,&val,&tv);
+          if (tv==Gnotype)
+            tv=mint;
+          vn = pushvar(var,flag,tv,val);/* can change ctxstack */
+          if (tv!=Gsmall || tree[val].f!=Fsmall)
+          {
+            v = ctxstack+vn;
+            if (descfindrules1(v->node,FC_default_check))
+              v->flag|=(1<<Cdefmarker);
+          }
         }
         break;
       }
-      genblock(val,n);
+      break;
+    case Fvararg:
+      if (decl==function && i==nb-1)
+        pushvar(tree[stack[i]].x,flag|(1<<Cvararg),Gvec,val);
+      else
+        die(n,"invalid variadic");
       break;
     default:
       die(n,"Incorrect node %s in function declaration",
@@ -309,31 +363,36 @@ int genblockdeclaration(int args, int n, int flag, int type, int *seq)
   return nb;
 }
 
-int genblockcheckargs(int seq, userfunc *ufunc)
+int genblockcheckargs(int sav1, int seq, userfunc *ufunc)
 {
   int i;
   gpfunc *gp=lfunc+currfunc;
-  context *fc=block+ufunc->bl;
   seq=addseqleft(newcall("_copyarg",-1),seq);
   for(i=ufunc->narg-1;i>=0;i--)
   {
-    ctxvar *v=fc->c+ufunc->sarg+i;
+    ctxvar *v=ctxstack+sav1+i;
+    gpdescarg *rule;
     int check=-1;
-    gpdescarg *rule=descfindrules1(v->node, FC_badtype);
-    if (rule->args[0].type==vartype(*v))
+    if (!(v->flag&(1<<Carg))) continue;
+    rule=descfindrules1(v->node, FC_badtype);
+    if (rule && rule->args[0].type==vartype(*v))
     {
       check=newcall("_badtype",newleaf(v->node));
       if (rule->type!=Gvoid)
-        check=newcall("if",
-            newnode(Flistarg,check,
-                     newcall("_err_type",newstringnode(gp->gpname,-1))));
+      {
+        int name=newstringnode(gp->gpname,-1);
+        int args = descfindrules1(name,findfuncdesc("_err_type")) ? name :
+                     newnode(Flistarg,name,v->node);
+        check=newcall("if", newnode(Flistarg,check, newcall("_err_type", args)));
+      }
     }
     if (v->initval>=0 && (v->flag&(1<<Cdefmarker)))
     {
       int n=newnode(Flistarg,
           newcall("_default_check",newleaf(v->node)),
-          newnode(Faffect,newleaf(v->node),v->initval));
+          newnode(Fassign,newleaf(v->node),v->initval));
       check=newcall("if", (check>=0)?newnode(Flistarg,n,check):n);
+      v->initval=-1;
     }
     if (check>=0)
       seq=addseqleft(check,seq);
@@ -365,7 +424,7 @@ void genblockdeffunc(int n)
   int seq=tree[n].y;
   const char *name=entryname(funcid);
   int args=tree[funcid].y;
-  int bl1,gltop=-1;
+  int bl1,gltop=-1,level=preclevel;
   int savcf,savnpv;
   int sav1,nodebl1;
   userfunc *ufunc;
@@ -402,6 +461,7 @@ void genblockdeffunc(int n)
   ufunc->narg=genblockdeclaration(args,n,(1<<Cuser)|(1<<Carg)|(1<<Ccompo),-1,NULL);
   /*Add return at the end*/
   seq=geninsertreturn(seq);
+  seq=genblockcheckargs(sav1+ufunc->sarg,seq,ufunc);
   /*generate block*/
   genblock(seq,-1);
   if (tree[funcid].x!=initnode)
@@ -411,18 +471,18 @@ void genblockdeffunc(int n)
     int i;
     for (i=sav1;i<s_ctx.n;i++)
     {
-      if (ctxstack[i].flag&(1<<Cglobal))
-        ctxstack[i].flag&=~(1<<Cglobal);
+      if (ctxstack[i].flag&(1<<Cfunction))
+        ctxstack[i].flag&=~(1<<Cfunction);
     }
   }
   copyctx(sav1,block+bl1);
   restorectx(sav1);
-  seq=genblockcheckargs(seq,ufunc);
   nodebl1=newnode(Fblock,bl1,seq);
   tree[n].y=nodebl1;
   /*restore globalvar*/
   currfunc=savcf;
   newprivvar=savnpv;
+  preclevel=level;
 }
 
 int newvectoridx(int var, int ind)
@@ -458,13 +518,13 @@ void genblockmatrixl(int n, int x, int y)
   bl2=newblock();
   seq=newcoeff(newleaf(vx),newleaf(vy),newleaf(var));
   aseq=newgetg(ret,newleaf(vsup), "t_VEC");
-  arg[0]=newnode(Faffect,var,newsmall(1));
+  arg[0]=newnode(Fassign,var,newsmall(1));
   arg[1]=newopcall(OPle,newleaf(var),vsup);
   arg[2]=newcoeff(newleaf(ret),newleaf(var),-1);
   arg[2]=geninsertvar(seq,arg[2]);
   genblock(arg[2],-1);
   arg[3]=newopcall(OPpp,newleaf(var),-1);
-  aseq=addseqright(aseq,newcall("for",listtoseq(arg,4)));
+  aseq=addseqright(aseq,newcall("for",stacktoargs(arg,4)));
   makeblocks(bl1,bl2,n,bseq,aseq,ret,savx);
 }
 void genblockaffmatrixl(int n, int x, int y, int z)
@@ -496,13 +556,13 @@ void genblockaffmatrixl(int n, int x, int y, int z)
   bl1=newblock();
   bl2=newblock();
   seq=newcoeff(newleaf(x),newleaf(vy),newleaf(var));
-  arg[0]=newnode(Faffect,var,newsmall(1));
+  arg[0]=newnode(Fassign,var,newsmall(1));
   arg[1]=newopcall(OPle,newleaf(var),vsup);
   arg[2]=newcoeff(newleaf(vz),newleaf(var),-1);
   arg[2]=geninsertvar(arg[2],seq);
   genblock(arg[2],-1);
   arg[3]=newopcall(OPpp,newleaf(var),-1);
-  aseq=newcall("for",listtoseq(arg,4));
+  aseq=newcall("for",stacktoargs(arg,4));
   makeblocks(bl1,bl2,n,bseq,aseq,vz,savx);
 }
 void genblockvector(int n, const char *typ)
@@ -518,13 +578,15 @@ void genblockvector(int n, const char *typ)
     arg[i]=geninsertvar(arg[i],newvectoridx(newleaf(ret),i));
   for(i=1;i<=nb;i++)
     genblock(arg[i],-1);
-  aseq=newcall("_makevec",listtoseq(arg,nb+1));
+  aseq=newcall("_makevec",stacktoargs(arg,nb+1));
   makeblock(newblock(),n,aseq,ret,s_ctx.n);
 }
 void genblockindex(int n)
 {
   int ret,z;
   int aseq;
+  if (n==GNORANGE || tree[n].f==Fsmall)
+    return;
   newdecl((1<<Cconst),Gsmall,-1,&ret);
   z=newleaf(n);
   tree[z]=tree[n];
@@ -566,45 +628,93 @@ void genblockmatrix(int n)
   }
   for(i=0;i<nb;i++)
     genblock(arg[i],-1);
-  aseq=newcall("_makevec",listtoseq(arg,nb));
+  aseq=newcall("_makevec",stacktoargs(arg,nb));
   makeblock(newblock(),n,aseq,ret,s_ctx.n);
 }
 
-void genblockprotoraw(int n, gpfunc *gp)
+void
+genblockvecaff(int n)
+{
+  int vars[STACKSZ];
+  int x=tree[n].x, y=tree[n].y;
+  int aseq, ret, i;
+  int nv=listtostack(tree[x].x,Fmatrixelts,vars,STACKSZ,"[,...,]=",n);
+  genblock(y,n);
+  newdecl((1<<Cauto)|(1<<Cconst),Gempty,-1,&ret);
+  aseq = geninsertvar(y,ret);
+  for(i=0; i<nv; i++)
+  {
+    genblock(vars[i],n);
+    vars[i] = newnode(Fassign,vars[i],newcoeff(newleaf(ret),newsmall(i+1),-1));
+  }
+  aseq = addseqright(aseq,stacktoseq(vars,nv));
+  makeblock(newblock(),n,aseq,ret,s_ctx.n);
+}
+
+void genblockproto(int n, gpfunc *gp)
 {
   int arg[STACKSZ];
-  const char *p=gp->proto.code;
+  const char *p=gp->proto.code ? gp->proto.code : "&DG"; /* 2.3 compat */
   char c;
   PPproto mod;
   int nb=genlistargs(n,arg,0,STACKSZ);
   int i=0;
-  while((mod=parseproto(&p,&c)))
+  while((mod=parseproto(&p,&c))!=PPend)
   {
+    if (i<nb && tree[arg[i]].f!=Fnoarg && (mod==PPdefault || mod==PPdefaultmulti))
+      mod=PPstd;
     switch(mod)
     {
-    case PPdefault:
-    case PPdefaultmulti:
-      if (i>=nb || tree[arg[i]].f==Fnoarg)
-        break;
-    case PPstd: /*Fall through */
-      if (c=='r' && i<nb)
+    case PPstd:
+      if (i>=nb)
+        die(n,"too few arguments");
+      switch(c)
       {
+      case 'r':
         if (tree[arg[i]].f==Fentry)
           tree[arg[i]].f=Fconst;
         else if (tree[arg[i]].f!=Fconst)
           die(n,"argument must be a raw string");
+        break;
+      case '&':
+        if (tree[arg[i]].f!=Frefarg)
+          die(arg[i],"missing &");
+        arg[i]=tree[arg[i]].x;
+      case '*': /*Fall through */
+        {
+          int a = detag(arg[i]);
+          if (tree[a].f==Fmatcoeff)
+          {
+            int l=getlvaluerr(a);
+            genblock(l,n);
+            ctxstack[getvarerr(l)].flag|=1<<Ccompo;
+          }
+          if (c=='*') multi++;
+          genblock(arg[i],n);
+          if (c=='*') multi--;
+          break;
+        }
+      default:
+        genblock(arg[i],n);
       }
       break;
-    default: /* Skip */
+    case PPstar:
+      genblock(tree[n].y,n);
+      i=nb-1;
+      break;
+    case PPauto: /* Skip */
       i--;
       break;
+    default:
+      break;
     }
     i++;
   }
 }
+
 /*
-  n: node
-  p: caller node
+n: node
+p: caller node
 
   Rule of the game:
   genblock must never be used twice on the same node.
@@ -612,9 +722,6 @@ void genblockprotoraw(int n, gpfunc *gp)
 
   genblock must not change p.
 */
-static long multi=0;
-extern int linecount;
-
 int newgetgvec(const char *t)
 {
   return newtag(newcall("_cgetg",
@@ -636,7 +743,6 @@ int genclosedlist(int nf, int n)
   return n;
 }
 
-
 static int has_mand_arg(const char *code)
 {
   PPproto mod;
@@ -678,63 +784,94 @@ void genblock(int n, int p)
       else
         genblockvector(n, "t_VEC");
     break;
-  case Faffect:
+  case Fassign:
     x=detag(x);
-    if (tree[x].f==Ffacteurmat)
-    {
-      int l=getlvalue(x);
-      genblock(l,n);
-      if (l>=0)
-      {
-        ctxvar *v=ctxstack+getvarerr(l);
-        v->flag|=1<<Ccompo;
-      }
-    }
-    if (tree[x].f==Ffacteurmat && tree[tree[x].y].f==FmatrixL)
-    {
-      int xx=tree[x].x;
-      int xy=tree[tree[x].y].x;
-      genblockaffmatrixl(n,xx,xy,y);
-    }
+    if (tree[x].f==Fvec)
+      genblockvecaff(n);
     else
     {
+      if (tree[x].f==Fmatcoeff)
+      {
+        int l=getlvaluerr(x);
+        int m=matindex_type(tree[x].y);
+        genblock(l,n);
+        ctxstack[getvarerr(l)].flag|=1<<Ccompo;
+        if (m==MAT_line)
+        {
+          int xx=tree[x].x;
+          int xy=tree[tree[tree[x].y].x].x;
+          genblockaffmatrixl(n,xx,xy,y);
+          break;
+        }
+      }
       genblock(x, n);
       genblock(y, n);
     }
     break;
   case Fmatrix:
-  case FmatrixR:
     genblock(x,n);
     genblock(y,n);
-    if (multi && x!=-1 && tree[x].f!=Fsmall)
-      genblockindex(x);
-    if (multi && y!=-1 && tree[y].f!=Fsmall)
-      genblockindex(y);
-    break;
-  case Ffacteurmat:
-    if ( tree[y].f==FmatrixL )
+    if (multi)
     {
-      y=tree[n].y=tree[y].x;
-      if (FC_matrixrow>=0)
+      genblockindex(tree[x].x);
+      genblockindex(tree[x].y);
+      if (y>=0)
       {
-        int nn;
-        genblock(x,n);
-        genblock(y,n);
-        nn=newcall("_[_,]",newnode(Flistarg,x,y));
-        tree[n]=tree[nn];
+        genblockindex(tree[y].x);
+        genblockindex(tree[y].y);
       }
-      else
-        genblockmatrixl(n,x,y);
     }
-    else
+    break;
+  case Fmatcoeff:
     {
       int nn;
+      int m=matindex_type(y);
+      int yx=tree[y].x, yy=tree[y].y;
       genblock(x,n);
       genblock(y,n);
-      nn=newcoeff(x,tree[y].x,tree[y].y);
+      switch(m)
+      {
+      case VEC_std:
+        nn=newcoeffsafe(x,tree[yx].x,-1);
+        break;
+      case MAT_std:
+        nn=newcoeffsafe(x,tree[yx].x,tree[yy].x);
+        break;
+      case MAT_line:
+        tree[n].y=yx;
+        if (FC_matrixrow>=0)
+          nn=newcall("_[_,]",newnode(Flistarg,x,tree[yx].x));
+        else
+        {
+          genblockmatrixl(n,x,tree[yx].x);
+          nn=n;
+        }
+        break;
+      case MAT_column:
+        nn=newcoeff(x,tree[yy].x,-1);
+        break;
+      case MAT_range:
+        {
+          int arg[5];
+          arg[0] = x;
+          arg[1] = tree[yx].x;
+          arg[2] = tree[yx].y;
+          if(yy<0)
+            nn=newnode(Ffunction,newentry("_[_.._]"),stacktoargs(arg,3));
+          else
+          {
+            arg[3] = tree[yy].x;
+            arg[4] = tree[yy].y;
+            nn=newnode(Ffunction,newentry("_[_.._,_.._]"),stacktoargs(arg,5));
+          }
+        }
+        break;
+      default:
+        die(-1,"Unknown matindex");
+      }
       tree[n]=tree[nn];
+      break;
     }
-    break;
   case Fcall:
     {
       int seq = newcall("_(_)",addlistleft(x,y));
@@ -743,20 +880,6 @@ void genblock(int n, int p)
       genblock(n,p);
     }
     break;
-  case Frefarg:
-    x=detag(x);
-    if (tree[x].f==Ffacteurmat)
-    {
-      int l=getlvalue(x);
-      genblock(l,n);
-      if (l>=0)
-      {
-        ctxvar *v=ctxstack+getvarerr(l);
-        v->flag|=1<<Ccompo;
-      }
-    }
-    genblock(x,n);
-    break;
   case Fentry:
     {
       int nf, nv = getvar(n);
@@ -769,7 +892,7 @@ void genblock(int n, int p)
         if (uf->flag&(1<<UFclosure))
         {
           /* We assume the closure contains the variable */
-          pushvar(n,(1<<Cuser)|(1<<Carg)|(1<<Ccompo)|(1<<Cclosed),Ggen,-1);
+          pushvar(n,(1<<Cuser)|(1<<Cfunction)|(1<<Carg)|(1<<Ccompo)|(1<<Cclosed),Ggen,-1);
           uf->narg++;
         }
         else
@@ -779,7 +902,7 @@ void genblock(int n, int p)
           * It is discarded in genblockdeffunc. This can be changed.
           */
           int iv=newquotenode(entryname(n),-1);
-          pushvar(n,(1<<Cuser)|(1<<Cglobal)|(1<<Cundeclared),Ggen,iv);
+          pushvar(n,(1<<Cuser)|(1<<Cfunction)|(1<<Cglobal)|(1<<Cundeclared),Ggen,iv);
           if (warn)
             warning(n,"variable undeclared");
         }
@@ -809,59 +932,19 @@ void genblock(int n, int p)
       tree[n].f=Ffunction;
     }
   case Ffunction:/*fall through*/
-    if (x==OPn)
-    {
-      if (tree[y].f==Fsmall)
-      {
-        tree[n]=tree[y];
-        tree[n].x=-tree[n].x;
-        break;
-      }
-      else if (tree[y].f==Fconst)
-      {
-        value_t *val = value + tree[y].x;
-        if (val->type==CSTsmall || val->type==CSTsmallreal)
-        {
-          tree[n]=tree[y];
-          val->val.small=-val->val.small;
-          break;
-        }
-      }
-    }
-    else if (x==OPtrans && tree[y].f==Fvec)
+    if (x==OPtrans && tree[y].f==Fvec)
     {
       if (tree[y].x==-1)
       {
         int z=newgetgvec("t_COL");
         tree[n]=tree[z];
-        break;
       }
       else
       {
         tree[n]=tree[y];
         genblockvector(n,"t_COL");
-        break;
       }
     }
-    /*FIXME: add true parsing of description*/
-    if (x>=OPss && x<=OPme)
-    {
-      int dy;
-      if (x<=OPpp)
-        dy=detag(y);
-      else
-        dy=detag(tree[y].x);
-      if (tree[dy].f==Ffacteurmat)
-      {
-        int l=getlvalue(dy);
-        genblock(l,n);
-        ctxvar *v = ctxstack+getvarerr(l);
-        v->flag|=1<<Ccompo;
-      }
-      multi++;
-      genblock(y,n);
-      multi--;
-    }
     else
     {
       int nf=findfunction(value[x].val.str);
@@ -870,7 +953,7 @@ void genblock(int n, int p)
       int nv = getvar(n);
       if (nf<0 && uf->flag&(1<<UFclosure) && nv==-1)
       {
-        pushvar(n,(1<<Cuser)|(1<<Carg)|(1<<Ccompo)|(1<<Cclosed),Ggen,-1);
+        pushvar(n,(1<<Cuser)|(1<<Cfunction)|(1<<Carg)|(1<<Ccompo)|(1<<Cclosed),Ggen,-1);
         uf->narg++;
         genblock(y,n);
       }
@@ -880,14 +963,16 @@ void genblock(int n, int p)
         tree[n]=tree[seq];
         genblock(n,p);
       }
+      else if (nf>=0 && lfunc[nf].iter)
+        genblockfunciter(n,lfunc+nf);
       else if (nf>=0 && lfunc[nf].spec>0)
         genblockfuncspec(n,p,lfunc+nf);
+      else if (nf>=0 && lfunc[nf].proto.code)
+        genblockproto(n,lfunc+nf);
+      else if (nf>=0 && x>=OPss && x<=OPme) /* For compatibility with PARI 2.3 */
+        genblockproto(n,lfunc+nf);
       else
-      {
-        if (nf>=0 && lfunc[nf].proto.code)
-          genblockprotoraw(n,lfunc+nf);
         genblock(y,n);
-      }
     }
     break;
   case Fdeffunc:
