@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2005  The PARI group.
+Copyright (C) 2000-2013  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -10,7 +10,7 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #include "config.h"
 #include <stdlib.h>
@@ -31,11 +31,12 @@ static int newfunc(const char *gpname)
   f->proto.cname=gpname;
   f->proto.code=NULL;
   f->proto.origin=NULL;
-  f->proto.help=NULL;
+  f->proto.help=-1;
   f->node=newnode(Ffun,nf,-1);
   functype(*f)=Gempty;
   funcmode(*f)=0;
   f->spec=GPpari;
+  f->iter=NULL;
   f->wrap=NULL;
   f->dsc=NULL;
   f->user=NULL;
@@ -59,12 +60,12 @@ int newuserfunc(const char *gpname)
   f->spec=GPuser;
   f->proto.origin=namelib;
   uf=f->user=(userfunc*) malloc(sizeof(*f->user));
-  stack_init(&uf->v,sizeof(*uf->var),(void *)&uf->var);
-  stack_init(&uf->g,sizeof(*uf->gcvar),(void *)&uf->gcvar);
+  stack_init(&uf->v,sizeof(*uf->var),(void **)&uf->var);
+  stack_init(&uf->g,sizeof(*uf->gcvar),(void **)&uf->gcvar);
   return nf;
 }
 
-int findfunction_len(const char *s, int n)
+int findfunction_len(const char *s, size_t n)
 {
   int i;
   for(i=0; i<s_func.n;i++)
@@ -101,13 +102,22 @@ int findfuncdescopt(const char *s)
   return n;
 }
 
-int findfunctype(char *s)
+int findfunctype(const char *s)
 {
   int nf=findfuncdesc(s);
   gpdescarg *da=descfindrules(0,NULL,lfunc+nf);
   return da->type;
 }
 
+int findfunctypeopt(const char *s)
+{
+  gpdescarg *da;
+  int nf=findfuncdescopt(s);
+  if (nf<0) return -1;
+  da=descfindrules(0,NULL,lfunc+nf);
+  return da->type;
+}
+
 int getfunc(const char *gpname)
 {
   int r=findfunction(gpname);
@@ -116,12 +126,46 @@ int getfunc(const char *gpname)
   return newfunc(strdup(gpname));
 }
 
+void genprec(FILE *fout, int level)
+{
+  if (level)
+  {
+    char buf[64];
+    sprintf(buf, "bitprec%d", level);
+    if (getvarbyname(buf)>=0)
+      fprintf(fout, "nbits2prec(%s)",buf);
+    else if (getvarbyname(buf+3)>=0)
+      fprintf(fout, "%s",buf+3);
+    else die(-1,"Internal error: prec not found!");
+  }
+  else
+    fprintf(fout, "prec");
+}
+
+void genbitprec(FILE *fout, int level)
+{
+  if (level)
+  {
+    char buf[64];
+    sprintf(buf, "bitprec%d", level);
+    if (getvarbyname(buf)>=0)
+      fprintf(fout, "%s",buf);
+    else if (getvarbyname(buf+3)>=0)
+      fprintf(fout, "prec2nbits(%s)",buf+3);
+    else die(-1,"Internal error: prec not found!");
+  }
+  else
+    fprintf(fout, "prec2nbits(prec)");
+}
 void genautoarg(FILE *fout, char c, int nerr)
 {
   switch(c)
   {
   case 'p':
-    fprintf(fout,"prec");
+    genprec(fout, preclevel);
+    break;
+  case 'b':
+    genbitprec(fout, preclevel);
     break;
   case 'P':
     fprintf(fout,"precdl");
@@ -140,6 +184,9 @@ int genarg(int nerr, FILE *fout, char c, int n, PPproto pr)
   case 'G':
     gencast(fout,n,Ggen);
     return 1;
+  case 'U':
+    gencast(fout,n,Gusmall);
+    return 1;
   case 'L':
   case 'P':
     gencast(fout,n,Gsmall);
@@ -147,10 +194,10 @@ int genarg(int nerr, FILE *fout, char c, int n, PPproto pr)
   case '&':
     if (tree[n].f!=Frefarg && pr == PPdefault)
       die(nerr,"Missing & for reference");
+  case '*': /* Fall through */
     fprintf(fout,"&");
     gencode(fout,n);
     return 1;
-  case '*':
   case 'W':
     gencode(fout,n);
     return 1;
@@ -286,21 +333,32 @@ void genuserfunc(FILE *fout, int n, int nf)
   gpfunc *gp=lfunc+nf;
   userfunc *ufunc=gp->user;
   context *fc=block+gp->user->bl;
-  nb=genlistargs(n,arg,0,ufunc->narg);
+  int narg = ufunc->narg;
+  if (narg && fc->c[ufunc->sarg+narg-1].flag&(1<<Cvararg))
+    nb=genlistargs(n,arg,0,STACKSZ);
+  else
+    nb=genlistargs(n,arg,0,narg);
   fprintf(fout,"%s(",gp->proto.cname);
-  for(j=0;j<ufunc->narg;j++)
+  for(j=0;j<narg;j++)
   {
     ctxvar *v=fc->c+ufunc->sarg+j;
     int t=vartype(*v);
     if (firstarg)
       fprintf(fout,", ");
     firstarg=1;
-    if (j<nb && arg[j]>=0 && arg[j]!=GNOARG)
+    if (v->flag&(1<<Cvararg))
+    {
+      int k;
+      for (k=j;k<nb;k++)
+        if (arg[k]==GNOARG) arg[k]=GNIL;
+      genfuncbydesc(fout,nb-j,arg+j,FC_tovec,nf);
+    }
+    else if (j<nb && arg[j]>=0 && arg[j]!=GNOARG)
       gencast(fout,arg[j],t);
-    else if (v->initval<0)
-      die(n,"a mandatory argument is missing");
     else if (v->flag&(1<<Cdefmarker))
       gencodenoarg(fout,t,n);
+    else if (v->initval<0)
+      die(n,"a mandatory argument is missing");
     else
       gencast(fout,v->initval,t);
   }
@@ -308,7 +366,7 @@ void genuserfunc(FILE *fout, int n, int nf)
   {
     if (firstarg) fprintf(fout,", ");
     else firstarg=1;
-    fprintf(fout,"prec");
+    genprec(fout, preclevel);
   }
   fprintf(fout,")");
 }
@@ -386,7 +444,7 @@ void genvarproto(FILE *fout, int n, int nerr)
     gencode(fout,n);
 }
 
-void genfuncproto(FILE *fout, int nf, char *sep, int nerr)
+void genfuncproto(FILE *fout, int nf, const char *sep, int nerr)
 {
   gpfunc *gp=lfunc+nf;
   int n=gp->node;
@@ -412,6 +470,8 @@ void gendecvar(FILE *fout, context *fc, int nerr)
 {
   int oldt=-1;
   int idx;
+  if (fc->s.n>0)
+    genlineno(fout, fc->c->node);
   for(idx=0;idx<fc->s.n;idx++)
   {
     ctxvar *v=fc->c+idx;
@@ -426,6 +486,8 @@ void gendecvar(FILE *fout, context *fc, int nerr)
     {
       gendecvarend(fout,oldt);
       genindent(fout);
+      if (idx<fc->s.n-1)
+        genlineno(fout, (v+1)->node);
       if (genfuncbydesc1(fout,v->node,FC_decl_base,nerr))
         die(nerr,"type `%s' not suitable for a variable",GPname(t));
       fprintf(fout, " ");
@@ -464,12 +526,16 @@ genprotoargs(FILE *fout, int nf, int firstarg, int nb, int *name)
     firstarg=1;
     switch(c)
     {
+    case 'b':
     case 'p':
     case 'P':
     case 'L':
     case 'n':
       fprintf(fout,"long");
       break;
+    case 'U':
+      fprintf(fout,"ulong");
+      break;
     case 'f':
       fprintf(fout,"long *");
       break;
@@ -486,20 +552,24 @@ genprotoargs(FILE *fout, int nf, int firstarg, int nb, int *name)
       break;
     case 'r':
     case 's':
-      fprintf(fout,"char *");
+      fprintf(fout,"const char *");
       break;
     default:
       die(err_func,"prototype letter `%c' not known",c);
     }
     if (name)
     {
-      if (i>=nb)
-        die(err_func,"too few arguments in lambda");
       fputs(" ",fout);
       if (c=='p')
         fprintf(fout,"prec");
+      else if (c=='b')
+        fprintf(fout,"bitprec");
       else
+      {
+        if (i>=nb)
+          die(err_func,"too few arguments in lambda");
         gencode(fout, name[i++]);
+      }
     }
   }
   if (name && i<nb)
@@ -544,7 +614,6 @@ void genprototype(FILE *fout, int nf, int kb)
         printnode(fout,v->initval);
         fprintf(fout,"*/");
       }
-
     }
   }
   s_ctx.n=savc;
@@ -559,7 +628,7 @@ void genprototype(FILE *fout, int nf, int kb)
 }
 
 static int
-genwrapargs(FILE *fout, int nf, int nb, int *name)
+genwrapargs(FILE *fout, int nf, int nb, int *name, long mode)
 {
   gpfunc *gp=lfunc+nf;
   int i = 0;
@@ -570,10 +639,9 @@ genwrapargs(FILE *fout, int nf, int nb, int *name)
   PPproto mod;
   while((mod=parseproto(&p,&c)))
   {
-    if (!firstarg) fprintf(fout,", ");
-    firstarg=0;
     switch(c)
     {
+    case 'b':
     case 'p':
     case 'P':
     case 'L':
@@ -595,12 +663,31 @@ genwrapargs(FILE *fout, int nf, int nb, int *name)
     default:
       die(err_func,"prototype letter `%c' not known",c);
     }
-    if (i>=nb)
-      die(err_func,"too few arguments in lambda");
-    if (c=='p')
-      fprintf(fout,"prec");
+    if (c=='p' || c=='b')
+    {
+      if (mode&(1<<Mprec))
+      {
+        if (!firstarg) fprintf(fout,", ");
+        firstarg=0;
+        if (c=='p')
+          genprec(fout, preclevel);
+        else
+          genbitprec(fout, preclevel);
+      }
+    }
     else
-      gencast(fout, name[i++], t);
+    {
+      int ot;
+      if (i>=nb)
+        die(err_func,"too few arguments in lambda");
+      if (!firstarg) fprintf(fout,", ");
+      firstarg=0;
+      ot = tree[name[i]].t;
+      tree[name[i]].t = t;
+      gencast(fout, name[i], ot);
+      tree[name[i]].t = ot;
+      i++;
+    }
   }
   if (name && i<nb)
     die(err_func,"too many arguments in lambda");
@@ -610,8 +697,9 @@ genwrapargs(FILE *fout, int nf, int nb, int *name)
 void genwrapper(FILE *fout, int nf, int wrap)
 {
   gpfunc *gp=lfunc+nf;
+  gpfunc *wr=lfunc+wrap;
   int t=functype(*gp), m=funcmode(*gp);
-  int has_prec=m&(1<<Mprec);
+  int has_prec=(m&(1<<Mprec)) && !(funcmode(*wr)&(1<<Mprec));
   int i,firstarg=1,d;
   int nerr=gp->user->defnode;
   int savc=s_ctx.n;
@@ -620,7 +708,6 @@ void genwrapper(FILE *fout, int nf, int wrap)
   int res;
   ctxvar *vres;
   context *fc=block+gp->user->bl;
-  gpfunc *wr=lfunc+wrap;
   gpdescarg *rule;
   if (!wr->proto.code)
     die(wr->node,"Wrapper not defined");
@@ -653,12 +740,12 @@ void genwrapper(FILE *fout, int nf, int wrap)
   {
     genfuncbydesc1(fout,vres->node,FC_decl_base,nerr);
     fprintf(fout," ");
-    if (genfuncbydesc1(fout,vres->node,FC_decl_ext,nerr));
+    if (genfuncbydesc1(fout,vres->node,FC_decl_ext,nerr))
       gencode(fout,vres->node);
     fprintf(fout," = ");
   }
   fprintf(fout,"%s(",gp->proto.cname);
-  firstarg=genwrapargs(fout, wrap, nb, stack);
+  firstarg=genwrapargs(fout, wrap, nb, stack, m);
   for (i=0,d=1;i<fc->s.n;i++)
   {
     ctxvar *v=fc->c+i;
@@ -711,6 +798,7 @@ void gendeffunc(FILE *fout, int n)
     die(n,"Internal error in gendeffunc : func %s not found",name);
   gp=lfunc+currfunc;
   gencomment(fout,funcid,0);
+  genlineno(fout, funcid);
   genprototype(fout,currfunc,1);
   t=functype(*gp);
   if (t!=Ggen && t!=Gsmall)
@@ -730,6 +818,7 @@ void gendefblock(FILE *fout, int n)
   int i;
   int m;
   int savc;
+  int level = preclevel;
   context *fc=block+b;
   savc=s_ctx.n;
   for(i=0;i<fc->s.n;i++)
@@ -748,8 +837,7 @@ void gendefblock(FILE *fout, int n)
   m=tree[n].m;
   if(!(m&(1<<Mbrace)))
   {
-    genindent(fout);
-    fprintf(fout,"{\n");
+    genindentline(fout,"{\n");
     indent++;
   }
   gendecvar(fout,fc,n);
@@ -759,10 +847,8 @@ void gendefblock(FILE *fout, int n)
   if(!(m&(1<<Mbrace)))
   {
     indent--;
-    genindent(fout);
-    fprintf(fout,"}");
-    if(!(m&(1<<Muntil)))
-      fprintf(fout,"\n");
+    genendbrace(fout,n);
   }
   s_ctx.n=savc;
+  preclevel = level;
 }
