@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2002  The PARI group.
+Copyright (C) 2002-2013  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -10,7 +10,7 @@ ANY WARRANTY WHATSOEVER.
 
 Check the License for details. You should have received a copy of it, along
 with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #include "config.h"
 #include <stdlib.h>
@@ -30,7 +30,9 @@ void gerepilelist(stack *s, stack *g, int savb)
     if (ctype[t]!=Vgen || ((v->flag&(1<<Cconst)) && v->val!=-1))
       continue;
     /*If i is affected or referenced*/
-    if (an[i].f!=AFaccess && an[i].f!=AFhide && an[i].f!=AFclone)
+    if (an[i].f==AFclone)
+      stack_remove_int(g,v->node);
+    else if (an[i].f!=AFaccess && an[i].f!=AFhide)
     {
       int j;
       /*search the first occurence of i*/
@@ -39,7 +41,7 @@ void gerepilelist(stack *s, stack *g, int savb)
           break;
       /*if the variable is local and the first occurence
         is an affectation, do nothing */
-      if (an[j].idx>=savb &&  an[j].f==AFaffect)
+      if (an[j].idx>=savb &&  an[j].f==AFassign)
         continue;
 
       /*else check if it is not already in the list*/
@@ -54,7 +56,7 @@ void gerepilelist(stack *s, stack *g, int savb)
     int i;
     fprintf(stderr,"/*");
     for(i=0;i<s->n;i++)
-      fprintf(stderr,"%s%s ",varstr(ctxstack[an[i].idx]),an[i].f==AFaffectcompo?"=[]":an[i].f==AFaffect?"=":"");
+      fprintf(stderr,"%s%s ",varstr(ctxstack[an[i].idx]),an[i].f==AFassigncompo?"=[]":an[i].f==AFassign?"=":"");
     fprintf(stderr,"*/\n");
   }
 }
@@ -69,7 +71,7 @@ void pilelistvar(int idx)
   if (v->initval>=0)
   {
     pilelist(v->initval);
-    newaff(AFaffect,idx);
+    newaff(AFassign,idx);
   }
 }
 
@@ -84,7 +86,7 @@ void pilelist(int n)
   y=tree[n].y;
   switch(tree[n].f)
   {
-  case Faffect:
+  case Fassign:
     if (tree[x].f!=Fentry)
       pilelist(x);
     pilelist(y);
@@ -148,7 +150,7 @@ void pilelist(int n)
     s_ctx.n=savc;
     break;
   default:
-    if (tree[n].f>=FneedENTRY || tree[n].f<0)
+    if (tree[n].f>=FneedENTRY)
       die(n,"Incorrect node %s in pilelist",funcname(tree[n].f));
     pilelist(x);
     pilelist(y);
@@ -165,7 +167,7 @@ void pileclean(int n)
   y=tree[n].y;
   switch(tree[n].f)
   {
-  case Faffect:
+  case Fassign:
     pileclean(x);
     pileclean(y);
     break;
@@ -224,7 +226,7 @@ void pileclean(int n)
     s_aff.n=sava;
     break;
   default:
-    if (tree[n].f>=FneedENTRY || tree[n].f<0)
+    if (tree[n].f>=FneedENTRY)
       die(n,"Incorrect node %s in pileclean",funcname(tree[n].f));
     pileclean(x);
     pileclean(y);
