@@ -1,16 +1,19 @@
 /*
-Copyright (C) 2000-2011  The PARI group.
+Copyright (C) 2000-2017  The PARI group.
 
 This file is part of the GP2C package.
 
-PARI/GP is free software; you can redistribute it and/or modify it under the
-terms of the GNU General Public License as published by the Free Software
-Foundation. It is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY WHATSOEVER.
+GP2C is free software; you can redistribute it and/or modify it under the terms
+of the GNU General Public License as published by the Free Software Foundation;
+either version 2 of the License, or (at your option) any later version.
 
-Check the License for details. You should have received a copy of it, along
-with the package; see the file 'COPYING'. If not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
+This program is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
+PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
+Street, Fifth Floor, Boston, MA 02110-1301 USA.*/
 
 #define TYPE
 #include "config.h"
@@ -22,25 +25,23 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include "header.h"
 
 FILE *yyin;
-int yyparse (void);
-void  yyrestart(FILE *new_file);
-void patchfunclist(void);
-void inittype(void);
-void initoperators(void);
 int linecount;
 int yydebug;
 static int optgen=0,opttree=0,opttype=0,optinfo=0;
 int optstrict=0;
 int optcleanvar=0;/*used in genbrace to suppress {} optimization*/
+int opthashline=0;
 int indentlevel=2;
+int preclevel=0;
 int do_warning=1;
 char *optprefix,*optsuffix;
-void printlistfunc(FILE *fout);
-void outputtype(FILE *fout);
-void cleancode(int n, int p);
-void cleanvar(int n);
-void initdesc(const char *s);
-void maketreeGRL(FILE *fout,int n);
+int debug;
+const char *nameparse, *namelib, *outfile;
+int lastpass;
+int autogc;
+int warn;
+int safecoeff;
+
 void dump(FILE *fout)
 {
   int i,j;
@@ -76,22 +77,24 @@ void
 init_compiler(void)
 {
   int n;
-  stack_init(&s_node,sizeof(*tree),(void *)&tree);
-  stack_init(&s_value,sizeof(*value),(void *)&value);
-  stack_init(&s_func,sizeof(*lfunc),(void *)&lfunc);
-  stack_init(&s_ctx,sizeof(*ctxstack),(void *)&ctxstack);
-  stack_init(&s_aff,sizeof(*affstack),(void *)&affstack);
-  stack_init(&s_bloc,sizeof(*block),(void *)&block);
-  stack_init(&s_comment,sizeof(*com),(void *)&com);
-  stack_init(&s_errors,sizeof(*errors),(void *)&errors);
-  stack_init(&s_Ctype,sizeof(*Ctype),(void *)&Ctype);
-  stack_init(&s_GPtype,sizeof(*GPtype),(void *)&GPtype);
-  stack_init(&s_Mmode,sizeof(*Mmode),(void *)&Mmode);
-  stack_init(&s_label,sizeof(*label),(void *)&label);
+  stack_init(&s_node,sizeof(*tree),(void **)&tree);
+  stack_init(&s_value,sizeof(*value),(void **)&value);
+  stack_init(&s_func,sizeof(*lfunc),(void **)&lfunc);
+  stack_init(&s_ctx,sizeof(*ctxstack),(void **)&ctxstack);
+  stack_init(&s_aff,sizeof(*affstack),(void **)&affstack);
+  stack_init(&s_bloc,sizeof(*block),(void **)&block);
+  stack_init(&s_comment,sizeof(*com),(void **)&com);
+  stack_init(&s_errors,sizeof(*errors),(void **)&errors);
+  stack_init(&s_Ctype,sizeof(*Ctype),(void **)&Ctype);
+  stack_init(&s_GPtype,sizeof(*GPtype),(void **)&GPtype);
+  stack_init(&s_Mmode,sizeof(*Mmode),(void **)&Mmode);
+  stack_init(&s_label,sizeof(*label),(void **)&label);
    /*Node 0 should be a (Gvoid)Fnoarg*/
   n=newnode(Fnoarg,-1,-1); tree[n].t=Gvoid;
    /*Node 1 should be a (Gnotype)Fnoarg*/
   n=newnode(Fnoarg,-1,-1); tree[n].t=Gnotype;
+   /*Node 2 should be a Fnorange*/
+  n=newnode(Fnorange,-1,-1); tree[n].t=Gsmall;
 }
 
 void
@@ -114,10 +117,18 @@ init_stdfunc(void)
   FC_copy=findfuncdesc("copy");
   FC_avma=findfuncdesc("_avma");
   Gpari_sp=findfunctype("_avma");
-  Gbptr=findfunctype("_diffptr");
   FC_low_stack_lim=findfuncdesc("_low_stack_lim");
+  FC_gc_needed=findfuncdescopt("_gc_needed");
   FC_gerepileall=findfuncdesc("_gerepileall");
-  FC_forprime_next=findfuncdescopt("_forprime_next");
+  FC_forprime_init=findfuncdescopt("_forprime_init");
+  if (FC_forprime_init>=0)
+    Gforprime=strtotype("forprime");
+  else
+  {
+    Gforprime=findfunctype("_diffptr");
+    FC_forprime_next=findfuncdesc("_forprime_next");
+  }
+  Gerror=findfunctypeopt("_iferr_error");
   FC_matrixrow=findfuncdescopt("_[_,]");
   FC_const_real=findfuncdescopt("_const_real");
   FC_call=findfuncdescopt("_(_)");
@@ -125,6 +136,12 @@ init_stdfunc(void)
   FC_proto_max_args=findfuncdescopt("_proto_max_args");
   max_args = FC_proto_max_args<0 ? 8 :
              lfunc[FC_proto_max_args].dsc->a[0].args[0].misc;
+  if (safecoeff && findfuncdescopt("_safecoeff")<0)
+  {
+    warning(-1,"gp2c option -C not supported by this PARI version");
+    safecoeff=0;
+  }
+  FC_norange=findfuncdescopt("_norange");
 }
 
 void
@@ -143,17 +160,22 @@ init_typedef(void)
 }
 
 void
-compile(FILE *fin, FILE *fout, char *nom)
+compile(FILE *fin, FILE *fout, const char *nom)
 {
-  char *descfile;
+  const char *descfile;
+  char *tmplib;
+  const char *lib;
   int startnode;
   int status;
   int i;
   init_compiler();
   linecount=1;
   nameparse=nom;
-  namelib=calloc(strlen(nameparse)+6,sizeof(*nameparse));
-  sprintf(namelib,"%s%s.so",nameparse[0]=='/'?"":"./",nameparse);
+  lib = genoutfile();
+  tmplib=(char*)calloc(strlen(lib)+6,sizeof(*lib));
+  sprintf(tmplib,"%s%s.so",lib[0]=='/'?"":"./",lib);
+  free((void*)lib);
+  namelib=tmplib;
   yyin=fin;
   currfunc=-1;
   initoperators();
@@ -197,7 +219,7 @@ compile(FILE *fin, FILE *fout, char *nom)
   if (yydebug) fprintf(stderr,"End of parsing\n");
 
   startnode=addseqleft(newnode(Ffunction,newentry("_initfunc"),-1),startnode);
-  gentoplevel(startnode,-1);
+  gentoplevel(startnode);
   startnode=addinitfunc(startnode);
   if (debug>1)
     printnode(stderr,startnode);
@@ -249,7 +271,7 @@ compile(FILE *fin, FILE *fout, char *nom)
   if (optinfo) dump(stderr);
   if (!optcleanvar)
     cleanvar(startnode);
-  cleancode(startnode,-1);
+  cleancode(startnode,-1,left);
   if (tree[startnode].f==Fseq && tree[startnode].y==GNIL)
     startnode=tree[startnode].x;
   switch(opttree)
@@ -273,7 +295,7 @@ compile(FILE *fin, FILE *fout, char *nom)
 void version(void)
 {
   printf("GP to C compiler version %s \n   targeted at PARI/GP %s\n",VERSION,PARI_VERSION);
-  printf("Copyright 2000-2011 The PARI Group\n");
+  printf("Copyright 2000-2017 The PARI Group\n");
   printf("GP2C is free software, covered by the GNU General Public License, and \n\
 you are welcome to change it and/or distribute copies of it under \n\
 certain conditions.  There is absolutely no warranty for GP2C.\n");
@@ -287,7 +309,9 @@ user option: \n\
  -o <file> :  Place output in file <file>. \n\
  -g : Generate automatic garbage collection code. \n\
  -iN: Set indentation level to N spaces (default 2). \n\
- -W : Output information about global variables. \n\
+ -W : Output warning about types and global variables usage. \n\
+ -C : Generate range checking code. \n\
+ -L : Generate #line directive for better C compiler messages. \n\
  -p <prefix>: Prefix user-defined symbol by <prefix> to avoid conflict. \n\
  -s <suffix>: Add suffix <suffix> to GP install names of functions. \n\
  -S : Assume strict declarations for functions. \n\
@@ -308,20 +332,22 @@ debugging options: \n\
 file.gp: file to be processed, default to stdin. \n\
 The generated C code is output to stdout unless the -o option is used \n\
  \n\
-You can automated the process of compiling with \n\
-the script gp2c-run. \n\
+See the script gp2c-run for an automated compilation process. \n\
 ",s);
 }
 int main(int argc, char **argv)
 {
   int c;
-  FILE *fin;
+  FILE *fin=stdin;
   FILE *fout=stdout;
+  const char *infile = "stdin";
   autogc=0;
+  safecoeff=0;
   warn=0;
   optprefix=NULL;
   optsuffix=NULL;
-  while((c=getopt(argc,argv,"gi:hflo:p:s:StvydTGVW"))!=-1)
+  outfile = NULL;
+  while((c=getopt(argc,argv,"gi:hflo:p:s:StvydCGLTVW"))!=-1)
   {
     switch(c)
     {
@@ -340,7 +366,7 @@ int main(int argc, char **argv)
       break;
     case 'l':
       {
-        char *descfile;
+        const char *descfile;
         init_compiler();
         if (!(descfile=getenv("GP2C_FUNC_DSC")))
           descfile=FUNCDSC_PATH;
@@ -350,7 +376,7 @@ int main(int argc, char **argv)
         exit(0);
       }
     case 'o':
-      fout=fopen(optarg,"w");
+      outfile=strdup(optarg);
       break;
     case 'p':
       optprefix=strdup(optarg);
@@ -380,9 +406,15 @@ int main(int argc, char **argv)
     case 'G':
       optgen=1-optgen;
       break;
+    case 'L':
+      opthashline=1-opthashline;
+      break;
     case 'V':
       optcleanvar=1-optcleanvar;
       break;
+    case 'C':
+      safecoeff=1-safecoeff;
+      break;
     case 'W':
       warn=1-warn;
       break;
@@ -397,20 +429,22 @@ int main(int argc, char **argv)
     usage(stderr,argv[0]);
     exit(1);
   }
-  if (argc==optind)
-    compile(stdin,fout,"stdin");
-  else
+  if (argc!=optind)
   {
-    char *nom=argv[optind];
-    char *nomf=nom;
-    if ((fin=fopen(nomf,"r"))==NULL)
+    infile = argv[optind];
+    if ((fin=fopen(infile,"r"))==NULL)
     {
       perror(argv[0]);
       exit(errno);
     }
-    compile(fin,fout,nom);
-    fclose(fin);
   }
+  if (outfile && ((fout=fopen(outfile,"w"))==NULL))
+  {
+    perror(argv[0]);
+    exit(errno);
+  }
+  compile(fin,fout,infile);
+  if (fin!=stdin) fclose(fin);
   if (fout!=stdout) fclose(fout);
   return 0;
 }
